var MainStageName = "MAIN";

function AppAssistant() {
    this.activeStages = {};
}

AppAssistant.prototype.setup = function() {
    Mojo.Log.info("Starting up the Evernote application");
    EvernoteContext.create();
};

AppAssistant.prototype.handleLaunch = function (launchParams) {
    Mojo.Log.info("[AppAssistant] handleLaunch");
    var stageProxy = this.controller.getStageProxy(MainStageName);
    var stageController = this.controller.getStageController(MainStageName);

    if (stageProxy) {
        Mojo.Log.info("[AppAssistant] Have a proxy");
        if (stageController) {

            Mojo.Log.info("[AppAssistant] Caling for sync");
            EvernoteContext.getSession().syncCaches(this.relaunchApp.bind(this), null, false);
        }
    }
    else {
        Mojo.Log.info("[AppAssistant] creating stage");
        var stageArguments = {
            name: MainStageName,
            lightweight: true,
            assistantName: "MainStageAssistant"
        };
        this.controller.createStageWithCallback(stageArguments, this._createStageCallback.bind(this), "card");
    }
};

AppAssistant.prototype._createStageCallback = function(stageController) {
    Mojo.Log.info("[AppAssistant] createStageCallback");
    var session = EvernoteContext.getSession();
    session.setAuthenticationHandlers(this.showDefaultScene.bind(this), this.showLoginScene.bind(this));
    session.start(null);
    Mojo.Log.info("[AppAssistant] createStageCallback - end");
};

AppAssistant.prototype.getMainStageAssistant = function() {
    return Mojo.Controller.getAppController().getStageController(MainStageName).assistant;
};

AppAssistant.prototype.relaunchApp = function() {
    Mojo.Log.info("[App Assistant] Relaunch App");
    var stageController = Mojo.Controller.getAppController().getStageController(MainStageName);
    stageController.activate();
};

AppAssistant.prototype.showDefaultScene = function() {
    Mojo.Log.info("[AppAssistant] showDefaultScene");
    var stageController = Mojo.Controller.getAppController().getStageController(MainStageName);
    stageController.assistant.showDefaultScene();
    stageController.activate();
};

AppAssistant.prototype.showLoginScene = function() {
    Mojo.Log.info("[AppAssistant] showLoginScene");
    var stageController = Mojo.Controller.getAppController().getStageController(MainStageName);
    stageController.assistant.showLoginScene();
    stageController.activate();
};

AppAssistant.prototype.openNoteStage = function(note, noteFilter) {

    stageName = note.guid || note.viewId;

    var stage = this.controller.getStageController(stageName);
    if (stage) {
        stage.activate();
    }
    else {
        var outerThis = this;
        var createStage = function(stageName) {
            var params = {
                name: stageName,
                lightweight: true,
                assistantName: "NoteStageAssistant"
            };
            var callback = function(stageController){
                stageController.assistant.showNote(note, noteFilter);
            };
            outerThis.controller.createStageWithCallback(params, callback);
            outerThis.activeStages[stageName] = true;
        };
        createStage(note.guid || note.viewId);
    }
};

AppAssistant.prototype.closeNoteStage = function(note) {
    var stageName = note.viewId;
	this.controller.closeStage(stageName);
	delete this.activeStages[stageName];
    //this.controller.getStageController(MainStageName).activate();
	Mojo.Log.info("[AppAssistant] closing note %s", stageName);
    
        // try to close any additional stages for that same note
    if (note.guid) {
	         for (var name in this.activeStages) {
				var stage = this.controller.getStageController(name);
	            if (stage && stage.assistant && stage.assistant.note && stage.assistant.note.guid && stage.assistant.note.guid == note.guid) {
					Mojo.Log.info("[AppAssistant] closing note %s", stageName);
		            this.controller.closeStage(name);
		            delete this.activeStages[name];
		        }
	         }
	     }
		
};

AppAssistant.prototype.handleCommand = function(event) {
    var stageController = this.controller.getStageController(MainStageName);

    if (stageController && event.type == Mojo.Event.command) {
        switch(event.command) {
			//ADDED NEXT 3
			case 'do-touchpad-back':
				this._handleTpBack();
				break;
            case 'do-logout':
                this._handleLogout();
                break;
            case 'do-help':
                this._handleHelp();
                break;
            case 'do-preferences':
                this._handlePreferences();
                break;
            case 'do-about':
                EvernoteContext.getNotifier().alertDialog($L("Evernote for webOS"),
                        $L("Copyright 2011, Evernote Corporation"), $L("OK"));
                break;
            case 'do-go-online':
                EvernoteContext.setManualDisconnect(false);
                break;
            case 'do-go-offline':
                EvernoteContext.setManualDisconnect(true);
                break;
        }
    }
};

AppAssistant.prototype._handleLogout = function() {
    var self = this;
    var onChoose = function(value) {
        if (value != "yes") {
            return;
        }
        for (var prop in self.activeStages) {
			if(name !== 'undefined')
            {
            	self.controller.closeStage(prop);
			}
        }
        self.activeStages = { };
        EvernoteContext.getSession().end();
        self.showLoginScene();
    };

    if (!EvernoteContext.getUpdateQueue().hasPendingTickets()) {
        onChoose("yes");
        return;
    }

    var choices = [ { label: $L("Yes"), value: "yes", type: "affirmative" },
        { label: $L("No"), value: "no", type: "negative" } ];

    var msg = $L("You have pending uploads that will be lost if you logout.  Do you want to continue logging out?");

    EvernoteContext.getNotifier().alertForm($L("Alert"), msg, onChoose, choices);
};

//ADDED NEXT 4
AppAssistant.prototype._handleTpBack = function() {
    var mainStage = this.controller.getStageController(MainStageName);
    mainStage.pushScene("note-list");
    mainStage.activate();
};

AppAssistant.prototype._handleHelp = function() {
    var mainStage = this.controller.getStageController(MainStageName);
    mainStage.pushScene("support");
    mainStage.activate();
};

AppAssistant.prototype._handlePreferences = function() {
    var mainStage = this.controller.getStageController(MainStageName);
    mainStage.pushScene("preferences");
    mainStage.activate();
};

AppAssistant.prototype.getCurrentScene = function() {
    return this.controller.getStageController(MainStageName).topScene();
};