function NoteEditAssistant(note){
    this.note = note;
    this.tags = null;
}

NoteEditAssistant.prototype.setup = function(){
    /**
     * Title Text Field
     */
    this.titleTextFieldModel = {
        title: this.note.title
    };
    this.titleTextFieldAttributes = {
        hintText: $L("Enter Note Title..."),
        textFieldName: 'title',
        modelProperty: 'title',
        disabledProperty: 'disabled',
        focus: true,
        modifierState: Mojo.Widget.capsLock,
        focusMode: Mojo.Widget.focusSelectMode,
        changeOnKeyPress: false,
        maxLength: 250,
        requiresEnterKey: false
    };
    this.controller.setupWidget('titleTextField', this.titleTextFieldAttributes, this.titleTextFieldModel);
    this.titleTextField = this.controller.get('titleTextField');

    /**
     * Change Attached File Button
     */
    this.changeAttachedFile = this._changeAttachedFile.bind(this);
    this.controller.listen('attachedFile', Mojo.Event.tap, this.changeAttachedFile);

    /**
     * Tag Text Field
     */
	this.tagsTextFieldModel = {
        tags: this.note.tags,
        tagNames: Tag.toCSV(this.note.tags)
    };
    if (this.note.tags == null) {
        this.tagsTextFieldModel.tags = [];
    }
	this.tagsTextFieldAttributes = {
        hintText: $L('Enter Tags...'),
        modelProperty: 'tagNames',
        textFieldMode: 'sentence-case',
        holdToEnable:true,
        focusMode: Mojo.Widget.focusAppendMode
    };
    this.controller.setupWidget('tagsTextField', this.tagsTextFieldAttributes, this.tagsTextFieldModel);
    this.tagsTextField = this.controller.get('tagsTextField');

    /**
     * Tag Selection Button
     */
    this.tagSelectionButtonModel = {
        disabled: false
    };
    this.tagSelectionButtonAttributes = {

    };
    this.controller.setupWidget('tagSelectionButton', this.tagSelectionButtonAttributes, this.tagSelectionButtonModel);
    this.toggleTagSelectionWidgetFunciton = this.toggleTagSelectionWidget.bindAsEventListener(this);
    this.controller.listen("tagSelectionButton", Mojo.Event.tap, this.toggleTagSelectionWidgetFunciton);
    this.tagSelectionButton = this.controller.get("tagSelectionButton");

    /**
     * Tag Selection Widget
     */
    this.tagSelectionWidgetModel = { myOpenProperty:false };
    this.controller.setupWidget('tagSelectionWidget', {property:'myOpenProperty'}, this.tagSelectionWidgetModel);
    this.tagSelectionWidget = this.controller.get("tagSelectionWidget");


    /**
     * Tag Selection List
     */
	  this.tagSelectionListModel = {
    };
    this.tagSelectionListAttributes = {
        itemTemplate: 'note-edit/list-tag',
        filterFunction: this.doTagsLoad.bind(this),
        renderLimit: 200
    };
    this.controller.setupWidget('tagSelectionList', this.tagSelectionListAttributes, this.tagSelectionListModel);
    this.tagSelectionList = this.controller.get('tagSelectionList');
    this.onTagSelectionFunction = this.onTagSelection.bindAsEventListener(this);
    this.controller.listen("tagSelectionList", Mojo.Event.listTap, this.onTagSelectionFunction);
    this.onFilterImmediatelyFunction = this.onFilterImmediately.bindAsEventListener(this);
    this.controller.listen("tagSelectionList", Mojo.Event.filterImmediate, this.onFilterImmediatelyFunction);

    /**
     * Notebook Selection List
     */
    this.notebookSelectionListModel = [];
    this.notebookSelectionModel = { selection: this.note.notebookGuid, choices: this.notebookSelectionListModel };
    this.controller.setupWidget('notebookSelectionList', {modelProperty:'selection'}, this.notebookSelectionModel);
    this.notebookSelectionList = this.controller.get('notebookSelectionList');
    EvernoteContext.getSession().NoteStore.listNotebooks(this.onNotebooksLoadSuccess.bind(this), this.onNotebooksLoadFailure.bind(this));

    /**
     * Content Text Area
     */
    this.contentTextAreaAttributes = {
		    hintText: $L("tap to edit content"),
        textFieldName: 'content',
        modelProperty: 'formattedContent',
        multiline: true,
        disabledProperty: 'disabled',
        focus: true,
        focusMode: ( this.note.isEmpty() ? Mojo.Widget.focusSelectMode : Mojo.Widget.focusInsertMode ),
        modifierState: Mojo.Widget.capsLock,
        limitResize: false,
        holdToEnable: false,
        changeOnKeyPress: true,
        textReplacement: false,
        requiresEnterKey: false
    };

	this.controller.setupWidget('contentTextArea', this.contentTextAreaAttributes, {value: ''});
	this.contentTextArea = this.controller.get('contentTextArea');
	
	// if(Mojo.Menu.styleEditItem.items[Mojo.Menu.styleEditItem.items.length -1].command != 'forecolor')
	// 		{
	// 			var foreColorMenu = {label:$L('Text Color'), command:'forecolor', checkEnabled:true, disabled: false};
	// 			Mojo.Menu.styleEditItem.items.push(foreColorMenu);
	// 		}	

	var menuManager = EvernoteContext.getMenuManager();                                         
 	var appMenuModel = menuManager.getBaseAppMenuModel();
	appMenuModel.richTextEditMenu = false;
	// appMenuModel.items.splice(0,0,Mojo.Menu.styleEditItem);
	// menuManager.setupAppMenu(this.controller, appMenuModel);
	
	appMenuModel.items.splice(0,0,Mojo.Menu.editItem);
	var textEditMenu = {label: "Format Text",
				items: [
					Mojo.Menu.boldItem,
					Mojo.Menu.italicItem,
					//Mojo.Menu.underlineItem,
					{label:$L('Text Color'), command:'forecolor', checkEnabled:true, disabled: false}]};
	appMenuModel.items.splice(1,0,textEditMenu);
	menuManager.setupAppMenu(this.controller, appMenuModel);
	 	

    /**
     * Command Menu Items
     */
    this.saveButtonAtt = {
        label: $L('Save'),
        command: 'do-note-save',
        icon: 'save'
    };
    this.cancelButtonAtt = {
        label: $('Cancel'),
        command: 'do-note-cancel',
        icon: 'cancel'
    };
    this.controller.setupWidget(Mojo.Menu.commandMenu, undefined, {items: [this.saveButtonAtt, this.cancelButtonAtt]});
    
    this.initialToDoCheckboxes = this._initializeToDoCheckboxes.bind(this);
    this.updateAttachedFileDisplay = this._updateAttachedFileDisplay.bind(this);
    this.scaleAttachedFileDisplay = this._scaleAttachedFileDisplay.bind(this);
	
	
	
	/**
     * Note Details Widget
     */
    this.noteDetailsWidgetModel = { myOpenProperty:false };
    this.controller.setupWidget('noteDetailsWidget', {property:'myOpenProperty'}, this.noteDetailsWidgetModel);
    this.noteDetailsWidget = this.controller.get("noteDetailsWidget");

	/**
     * Details Button
     */
	this.toggleNoteDetailsWidgetFunction = this.toggleNoteDetailsWidget.bindAsEventListener(this);
    this.controller.listen('detailsButton', Mojo.Event.tap, this.toggleNoteDetailsWidgetFunction);
    this.detailsButton = this.controller.get('detailsButton');
    this.detailsButton.update($L("Details"));
};

NoteEditAssistant.prototype.cleanup = function(event){
	this.controller.stopListening('attachedFile', Mojo.Event.tap, this.changeAttachedFile);
    this.controller.stopListening("tagSelectionButton", Mojo.Event.tap, this.toggleTagSelectionWidgetFunciton);
    this.controller.stopListening("tagSelectionList", Mojo.Event.listTap, this.onTagSelectionFunction);
    this.controller.stopListening("tagSelectionList", Mojo.Event.filterImmediate, this.onFilterImmediatelyFunction);
	this.controller.stopListening('detailsButton', Mojo.Event.tap, this.toggleNoteDetailsWidgetFunction);
};

NoteEditAssistant.prototype.activate = function(event){
    this.titleTextField.focus();
    if (! this.note.isEmpty()) {
        var xml = this.note.getFormattedContent();
        this.contentTextArea.update(xml);
        this.initialToDoCheckboxes.defer();
    }

    if (event && event.filename && event.returnValue) {
        this.note.file = {
            fullPath: event.filename
        };
    }

    this._updateAttachedFileDisplay();
};

NoteEditAssistant.prototype.deactivate = function(event){
};

NoteEditAssistant.prototype.handleTextColorMenu = function() {
	var controller = this.controller;
	var onselect = function(color){
		controller.document.execCommand("ForeColor", false, color);
	};
	var colorPicker = new ColorPickerDialogAssistant(this, onselect);
	colorPicker.show();
};

NoteEditAssistant.prototype.tagTextFieldLeave = function(event) {
	var focusNode = null;
	if(typeof this.controller != 'undefined' && 
		typeof this.controller.window != 'undefined' && 
		typeof this.controller.window.document != 'undefined'){
			focusNode = this.controller.window.document.querySelector(':focus');
	}
	if(event.keyCode == Mojo.Char.enter && focusNode && Mojo.View.isTextField(focusNode)){
		var isOpen = this.tagSelectionWidget.mojo.getOpenState();
	    if (isOpen) {
			this.closeTagSelectionWidget();
		}
		this.closeNoteDetailsWidget();
		event.cancelBubble=true;
	}
};

NoteEditAssistant.prototype.toggleNoteDetailsWidget = function(event) {
	if(this.noteDetailsWidget.mojo.getOpenState() == true){
		this.closeNoteDetailsWidget();
	}
	else{
		this.openNoteDetailsWidget();
	}
};

NoteEditAssistant.prototype.openNoteDetailsWidget = function(event) {
	this.controller.document.addEventListener("keyup", this.tagTextFieldLeave.bind(this), true);
	this.noteDetailsWidget.mojo.setOpenState(true);
};

NoteEditAssistant.prototype.closeNoteDetailsWidget = function(event) {
	this.controller.document.removeEventListener("keyup", this.tagTextFieldLeave.bind(this), true);
	this.noteDetailsWidget.mojo.setOpenState(false);
};

NoteEditAssistant.prototype.toggleTagSelectionWidget = function() {
    var isOpen = this.tagSelectionWidget.mojo.getOpenState();
          
    if (isOpen) {
        this.closeTagSelectionWidget();
    }
    else {
        this.openTagSelectionWidget();
    }
};

NoteEditAssistant.prototype.openTagSelectionWidget = function() {
	this.doTagsLoad("", this.tagSelectionList, 0, 50);
	this.tagSelectionButton.className += " pressed";
	this.tagSelectionWidget.mojo.setOpenState(true);
};

NoteEditAssistant.prototype.closeTagSelectionWidget = function() {
	this.tagSelectionButton.className = "";
	this.tagSelectionWidget.mojo.setOpenState(false);
	this.tagSelectionList.mojo.setLength(0);
	this.tagSelectionList.mojo.close();
};


NoteEditAssistant.prototype.onFilterImmediately = function(event) {
    if (event.filterString)
        this.titleTextField.style.display = "none";
    else
        this.titleTextField.style.display = "block";
};

NoteEditAssistant.prototype.doTagsRender = function(filter, listWidget, offset, limit) {
    Mojo.Log.info("[NoteEdit]doTagsRender");
    var filtered = [];
    for (var j = 0; j < this.tags.length; j++) {
        if (this.tags[j].name.toLowerCase().indexOf(filter.toLowerCase()) == 0) {
            var name = this.tags[j].name;
            filtered.push( { name: name, guid: this.tags[j].guid } );
        }
    }

    var slice = [];
    for (j = 0; j < limit; j++) {
        if (filtered[offset + j])
            slice[j] = filtered[offset + j];
    }

    listWidget.mojo.setLength(filtered.length);
    listWidget.mojo.noticeUpdatedItems(offset, slice);
    listWidget.mojo.setCount(filtered.length);
    Mojo.Log.info("[NoteEdit]doTagsRender - end");
};

NoteEditAssistant.prototype.doTagsLoad = function(filter, listWidget, offset, limit) {
    Mojo.Log.info("[NoteEdit] doTagsLoad");
    if (! this.tags) {
        Mojo.Log.info("[NoteEdit] Requesting tags");        
        EvernoteContext.getSession().NoteStore.listTags(
                this._onTagsLoadSuccess.bind(this, filter, listWidget, offset, limit),
                this._onTagsLoadFailure.bind(this));
    }
    else {
        Mojo.Log.info("[NoteEdit] doTagsLoad - has Tags");
        this.doTagsRender(filter, listWidget, offset, limit);
    }
    Mojo.Log.info("[NoteEdit] doTagsLoad - end");
};

NoteEditAssistant.prototype._onTagsLoadSuccess = function(filter, listWidget, offset, limit, tagsJSON) {
    Mojo.Log.info("[NoteEdit] _onTagsLoadSuccess");
    this.tags = tagsJSON.list;
    this.doTagsRender(filter, listWidget, offset, limit);
    Mojo.Log.info("[NoteEdit] _onTagsLoadSuccess - end");
};

NoteEditAssistant.prototype._onTagsLoadFailure = function() {
    Mojo.Log.info("[NoteEdit] _onTagsLoadFailure");
    EvernoteContext.getNotifier().alertDialog($L("Tag Load Error"), $L("An error has occured loading Tags."), $L("OK"), this.controller);
    Mojo.Log.info("[NoteEdit] _onTagsLoadFailure");    
};

NoteEditAssistant.prototype.onTagSelection = function(event) {
    var selectedTags = Tag.fromCSV(this.tagsTextFieldModel.tagNames);
    var selection = EvernoteContext.getSession().NoteStore.getTag(event.item.guid);
    var duplicate = false;

    for (var i = 0; i < selectedTags.length; i++) {
        if (selection.name.toUpperCase() == selectedTags[i].toUpperCase()) {
            duplicate = true;
            break;
        }
    }

    if (!duplicate) {
        selectedTags.push(selection.name);
        this.tagsTextFieldModel.tagNames = selectedTags.join(", ");
        this.controller.modelChanged(this.tagsTextFieldModel);
    }
    this.toggleTagSelectionWidget();
};

NoteEditAssistant.prototype.onNotebooksLoadSuccess = function(notebooksJSON) {
    var notebooks = notebooksJSON.list;

    for (var i = 0; i < notebooks.length; i++) {
        this.notebookSelectionListModel.push({label:$L(notebooks[i].name), value:notebooks[i].guid , secondaryIcon:'status-available', disabled:false});
    }

    if (this.note.notebookGuid == null) {
        this.note.notebookGuid = EvernoteContext.getSession().currentUser.defaultNotebookGuid;
    }

    this.notebookSelectionModel.selection = this.note.notebookGuid;
    this.controller.modelChanged(this.notebookSelectionModel);
};

NoteEditAssistant.prototype.onNotebooksLoadFailure = function() {
    Mojo.Log.error("Error loading notebooks for user!");
};

NoteEditAssistant.prototype.save = function(event) {
    this.note.title = this.titleTextFieldModel.title || $L("Untitled note");
    this.note.tags = [];

    this.tagsTextField.mojo.blur();
    var selectedTagNames = Tag.fromCSV(this.tagsTextFieldModel.tagNames);
    for (var i = 0; i < selectedTagNames.length; i++) {
        var selectedTag = EvernoteContext.getSession().NoteStore.getTagByName(selectedTagNames[i]);

        if (selectedTag == null) {
            selectedTag = new Tag(null, selectedTagNames[i]);
        }
        this.note.tags.push(selectedTag);
    }

    this.note.notebookGuid = this.notebookSelectionModel.selection;

    var todoCheckboxes = this.contentTextArea.select("input.en-todo");
    if (todoCheckboxes.length != this.note.todos.keys().length) {
        this.note.unsyncedTodos = true;
        this.note.todos = new Hash();
    }

    var todoIndex = 0;
    var self = this;
    todoCheckboxes.each(function(element) {
        var todo = "todo-" + todoIndex;
        element.writeAttribute("id",todo);
        if (element.checked != self.note.todos.get(todo)) {
            self.note.unsyncedTodos = true;
            self.note.todos.set(todo, element.checked);
        }

        todoIndex = todoIndex + 1;
    });

    this.note.formattedContent =  this.contentTextArea.innerHTML;
    this.note.highlightedFormattedContent = this.note.formattedContent;

    this.note.updated = new Date().getTime();

    if (this.note.isNew()) {
        this.note.created = this.note.updated;
        EvernoteContext.getUpdateQueue().createNote(
                this.onCreateSuccess.bind(this),
                this.onCreateFailure.bind(this),
                this.note);
    }
    else {
        EvernoteContext.getUpdateQueue().updateNote(
                this.onSaveSuccess.bind(this),
                this.onSaveFailure.bind(this),
                this.note);
    }
};

NoteEditAssistant.prototype.cancel = function(event){
	//Mojo.Log.error(this.contentTextArea.innerHTML);
    var onChoose = function(value) {
        if (value != "yes") {
            return;
        }

        this.controller.stageController.assistant.popOrCloseNoteScene();
    }.bind(this);

    var choices = [ { label: $L("Discard"), value: "yes", type: "negative" },
                    { label: $L("Cancel"), value: "no", type: "dismiss" } ];

    var msg = $L("Are you sure you want to discard any changes made to this note ?");

    EvernoteContext.getNotifier().alertForm($L("Alert"), msg, onChoose, choices);
};

NoteEditAssistant.prototype.onCreateSuccess = function(){
	Mojo.Log.info("[NoteEditAssistant] onCreateSuccess: entered");
    

    // if we saved via the "back" gesture, the scene is already popped (and the controller is out of scope)
    if (this.controller && this.controller.stageController)
        this.controller.stageController.assistant.popOrCloseNoteScene();

    EvernoteContext.getNotifier().banner($L("Note saved to pending uploads."), "evernote");
	ModelRegistry.fireEvent(this.note, ModelRegistry.Event.onCreate);
};

NoteEditAssistant.prototype.onCreateFailure = function(transport) {
    EvernoteContext.getErrorManager().showNoteCreateError(transport, this.controller);
};

NoteEditAssistant.prototype.onSaveSuccess = function() {
   

    // if we saved via the "back" gesture, the scene is already popped (and the controller is out of scope)
    if (this.controller && this.controller.stageController)
        this.controller.stageController.assistant.popOrCloseNoteScene();

    if (! EvernoteContext.isConnected()) {
        Mojo.Controller.getAppController().assistant.getMainStageAssistant().showUpdateQueueScene();
        EvernoteContext.getNotifier().banner($L("Note saved to pending uploads."), "evernote");
    }
    else {
        EvernoteContext.getNotifier().banner($L("Note saved to pending uploads."), "evernote");
    }
	ModelRegistry.fireEvent(this.note,ModelRegistry.Event.onUpdate);
};

NoteEditAssistant.prototype.onSaveFailure = function(transport) {
    EvernoteContext.getErrorManager().showNoteUpdateError(transport, this.controller);
};

NoteEditAssistant.prototype.handleCommand = function(event) {
	if(event.type == Mojo.Event.command) {
		switch(event.command)
		{
			//ADDED NEXT 3
			case 'do-touchpad-back':
				this.save();
			break;
			case 'do-note-save':
				this.save();
			break;
			case 'do-note-cancel':
				this.cancel();
			break;
			case 'forecolor':
				this.handleTextColorMenu(this);
				break;
		}
	}
  else if (event.type == Mojo.Event.back) {
      this.save();
  }
	else if (event.type == Mojo.Event.commandEnable) {
		if (event.command === 'forecolor') {
			var focusNode = this.controller.window.document.querySelector(':focus');
			//Mojo.View.getFocusedElement(this.controller.activeScene().sceneElement);
			if(!(focusNode && Mojo.View.isRichTextField(focusNode))) {
				event.preventDefault();
			}
			event.stopPropagation();
		}
	}
};

NoteEditAssistant.prototype._initializeToDoCheckboxes = function() {

    var unsyncedTodos =  this.note.unsyncedTodos;
    var todoCheckboxes = this.contentTextArea.select("input.en-todo");
    var todoIndex = 0;
    var self = this;

    todoCheckboxes.each(function(element) {
      element.enable();
      var todo = "todo-" + todoIndex;
      element.writeAttribute("id",todo);
      if (unsyncedTodos) {
          element.checked = self.note.todos.get(todo);
      }

      todoIndex = todoIndex + 1;
    });

};

NoteEditAssistant.prototype._updateAttachedFileDisplay = function() {

    this.attachedFile = this.controller.get('attachedFile');
    if (this.note.hasFile()) {
		if(this.note.fileSource <= Note.ATTACHED_FILE_SRC_PHOTO)
		{
        	var imageHTML = "<img id=\"attachedFileIcon\" src=\"" + this.note.getFileFullPath() + "\" />";
		}
		else
		{
			var imageHTML = "<img id=\"attachedFileIcon\" src=\""+ "./images/file-generic.png" + "\" />";
		}

        this.attachedFileIconSpace = this.controller.get('attachedFileIconSpace');
        this.attachedFileIconSpace.update(imageHTML);
        this.attachedFile.show();
        this.scaleAttachedFileDisplay.defer();
    } else {
        this.attachedFile.hide();
    }

};

NoteEditAssistant.prototype._scaleAttachedFileDisplay = function() {

    if (this.note.hasFile()) {
        this.attachedFileIcon = this.controller.get('attachedFileIcon');
        var width = this.attachedFileIcon.getWidth();
        var height = this.attachedFileIcon.getHeight();
        var ratio = width / height;
        var scaledWidth = width;
        var scaledHeight = height;

        if (width >= height) {
            if (width > 32) {
                scaledWidth = 32;
                scaledHeight = 32 / ratio;
            }
        } else {

            if (height > 32) {
                scaledHeight = 32;
                scaledWidth = 32 * ratio;
            }
        }

        this.attachedFileIcon.width = scaledWidth;
        this.attachedFileIcon.height = scaledHeight;
		
		var label = (this.note.fileSource == Note.ATTACHED_FILE_SRC_SNAPSHOT ? $L("Retake Snapshot") : $L("Reselect File"));
        this.attachedFileLabel = this.controller.get('changeAttachedFileLabel');
        this.attachedFileLabel.update(label);
    }

};

NoteEditAssistant.prototype._changeAttachedFile = function() {

    if (this.note.fileSource >= Note.ATTACHED_FILE_SRC_PHOTO) {
        var self = this;
        var onSelect = function(file) {
            self.note.file = file;
			if(file.attachmentType && file.attachmentType == 'image')
		    {
				self.note.fileSource = Note.ATTACHED_FILE_SRC_PHOTO;
			}
			else{
				self.note.fileSource = Note.ATTACHED_FILE_SRC_FILE;
			}
        }.bind(this);

       	var user = EvernoteContext.getSession().currentUser;

		if (user && user.privilege && user.privilege <= 1) {
			var params = {
		        onSelect: onSelect,
		        actionType: "attach",
		        kinds: ["image","audio","video","file"],
				extensions: ['gif', 'jpeg', 'jpg', 'mpg', 'png', 'wav', 'mpeg', 'amr', 'pdf'],
		        defaultKind: "image"
		    };
	    }
	    else {
	        var params = {
		        onSelect: onSelect,
		        actionType: "attach",
		        kinds: ["image","audio","video","file"],
		        defaultKind: "image"
		    };
	    }

        Mojo.FilePicker.pickFile(params, this.controller.stageController);
    } else if (this.note.fileSource == Note.ATTACHED_FILE_SRC_SNAPSHOT) {
        this.controller.stageController.pushScene({
            appId: "com.palm.app.camera",
            name: "capture"
        }, {
            sublaunch: true
        });
    }

};