function NoteViewAssistant(note, noteFilter) {
    this.note = note;
    this.noteFilter = noteFilter;
    this.session = EvernoteContext.getSession();
    this.user = this.session.currentUser;
    this.noteStore = this.session.NoteStore;
    this.geoNameRetrieved = false;
}

NoteViewAssistant.prototype.setup = function() {
    this.titleValueLabel = this.controller.get('titleValueLabel');

    this.spinner = this.controller.get('header-spinner');

    this.noteDetailsContainer = this.controller.get('noteDetailsContainer');

	this.notebookContainer = this.controller.get('notebookContainer');
    this.notebookLabel = this.controller.get('notebookLabel');
    this.notebookLabel.update($L('Notebook:'));
    this.notebookValueLabel = this.controller.get('notebookValueLabel');

    this.createdLabel = this.controller.get('createdLabel');
    this.createdLabel.update($L('Created:'));
    this.createdValueLabel = this.controller.get('createdValueLabel');

    this.updatedLabel = this.controller.get('updatedLabel');
    this.updatedLabel.update($L('Updated:'));
    this.updatedValueLabel = this.controller.get('updatedValueLabel');

	this.tagsContainer = this.controller.get('tagsContainer');
    this.tagListWidgetLabel = this.controller.get('tagsWidgetLabel');
    this.tagListWidgetLabel.update($L('Tags:'));
    this.tagListWidget = this.controller.get('tagsWidget');

    this.gpsLocation = this.controller.get('gpsLocation');

    this.locationLabel = this.controller.get('locationLabel');
    this.locationLabel.update($L('Created at:'));
    this.locationLink = this.controller.get('locationLink');
    this.locationLinkText = this.controller.get('locationLinkText');

	this.sourceContainer = this.controller.get('sourceContainer');

    this.sourceLabel = this.controller.get('sourceLabel');
    this.sourceLabel.update($L('Source:'));
    this.sourceLink = this.controller.get('sourceLink');
    this.sourceLinkText = this.controller.get('sourceLinkText');

    /**
     * Details Widget
     */
    this.detailsWidgetModel = { myOpenProperty:false };
    this.controller.setupWidget('detailsWidget', {property:'myOpenProperty'}, this.detailsWidgetModel);
    this.detailsWidget = this.controller.get('detailsWidget');

    this.toggleDetailsWidgetFunction = this.toggleDetailsWidget.bindAsEventListener(this);
    this.controller.listen('detailsWidget', Mojo.Event.tap, this.toggleDetailsWidgetFunction);

    /**
     * Details Button
     */
    this.controller.listen('detailsButton', Mojo.Event.tap, this.toggleDetailsWidgetFunction);
    this.detailsButton = this.controller.get('detailsButton');
    this.detailsButton.update($L("Details"));

	/**
     * Attachments Widget
     */
    this.filesWidgetModel = { myOpenProperty:false };
    this.controller.setupWidget('filesWidget', {property:'myOpenProperty'}, this.filesWidgetModel);
    this.filesWidget = this.controller.get('filesWidget');

    this.toggleFilesWidgetFunction = this.toggleFilesWidget.bindAsEventListener(this);
    this.controller.listen('filesWidget', Mojo.Event.tap, this.toggleFilesWidgetFunction);

    /**
     * Attachments Button
     */
    this.controller.listen('filesButton', Mojo.Event.tap, this.toggleFilesWidgetFunction);
    this.filesButton = this.controller.get('filesButton');
    this.filesButton.update($L("Files"));


	var listAttributes = {
        itemTemplate: 'note-view/attachment',
        disabledProperty: 'disabled'
    };
    this.attachmentsListModel = {
		items:  [],
        disabled: false
    };

    this.controller.setupWidget('fileList', listAttributes, this.attachmentsListModel);
    this.tappedFunction = this._handleAttachment.bindAsEventListener(this);
    this.controller.listen('fileList', Mojo.Event.listTap, this.tappedFunction);

	this.downloadProgressAttributes = {
	        title: '',
	        modelProperty: "progress",
	        icon: "test-progress"
    };
    this.download = {
        progress: 0
        // iconPath: "../images/list_button_add.png",
        //         status: Downloader.STATUS_NONE,
        //         buttonLabel: "Cancel",
        //         buttonClass: "negative"
    };
    this.controller.setupWidget("downloadProgress",
        this.downloadProgressAttributes,
        this.download
    );

	// 	this.downloadProgressTapped = this._downloadProgressTapped.bind(this);
	// 	this.controller.listen("downloadProgress",Mojo.Event.tap, this.downloadProgressTapped);

    /**
     * Content Text Area
     */
    this.contentTextAreaAttributes = { };
    this.controller.setupWidget("contentTextArea", this.contentTextAreaAttributes, {});
    this.contentTextArea = this.controller.get('contentTextArea');

    /**
     * Content Scroller
     */
    this.contentScrollerAttributes = { mode: 'dominant' };
    this.controller.setupWidget("contentScroller", this.contentScrollerAttributes, {});
    this.contentScroller = this.controller.get("contentScroller");

    /**
     * Command Menu
     */
    this.refreshButtonAtt = {
        label: $L('Refresh'),
        icon: 'refresh',
		disabled: true,
        command: 'do-note-refresh'
    };
    this.deleteButtonAtt = {
        label: $L('Delete'),
        icon: 'delete',
		disabled: true,
        command: 'do-note-delete'
    };
    this.emailButtonAtt = {
        label: $L('Email'),
        icon: 'forward-email',
        command: 'show-note-email'
    };
    this.sendButtonAttributes = {
        label: $L('Edit'),
        icon: 'edit',
		disabled: true,
        command: 'show-note-edit'
    };
    this.controller.setupWidget(Mojo.Menu.commandMenu, undefined, this.cmdMenuModel = {
        items: [
            this.sendButtonAttributes,
            this.refreshButtonAtt,
			// this.emailButtonAtt,
            this.deleteButtonAtt
        ]});

    this.initialToDoCheckboxes = this._initializeToDoCheckboxes.bind(this);
    this.handleToDoCheckboxChange = this._handleToDoCheckboxChange.bind(this);
    this.deinitialToDoCheckboxes = this._deinitializeToDoCheckboxes.bind(this);
    this.initializeAttachments = this._initializeAttachments.bind(this);
	this.initializeImages = this._initializeImages.bind(this);
	this.initializeDecryption = this._initializeDecryption.bind(this);
	this.handleImage = this._handleImage.bind(this);
    this.deinitializeImages = this._deinitializeImages.bind(this);
	this.deinitializeDecryption = this._deinitializeDecryption.bind(this);
    this.handleAttachment = this._handleAttachment.bind(this);
	this.handleDecryption = this._handleDecryption.bind(this);
    this.deinitializeAttachments = this._deinitializeAttachments.bind(this);

    EvernoteContext.getMenuManager().setupBaseAppMenu(this.controller);
};

NoteViewAssistant.prototype.cleanup = function(event) {
    this.controller.stopListening('detailsWidget', Mojo.Event.tap, this.toggleDetailsWidgetFunction);
    this.controller.stopListening('detailsButton', Mojo.Event.tap, this.toggleDetailsWidgetFunction);
    //this.controller.stopListening('downloadProgress', Mojo.Event.tap, this.downloadProgressTapped);
	this.controller.stopListening('filesButton', Mojo.Event.tap, this.toggleFilesWidgetFunction);
	this.controller.stopListening('filesWidget', Mojo.Event.tap, this.toggleFilesWidgetFunction);
    this.deinitialToDoCheckboxes();
    this.deinitializeImages();
	this.deinitializeDecryption();
    this.deinitializeAttachments();
};

NoteViewAssistant.prototype._reloadNoteMetadata = function(onSuccess, onFailure) {
    this.noteStore.getNote(onSuccess, onFailure, this.note.guid, false);
};

NoteViewAssistant.prototype._reloadNoteContent = function(onSuccess, onFailure) {

    if (EvernoteContext.getUpdateQueue().hasTicketForNote(this.note) || ! EvernoteContext.isConnected()) {
        if (this.note.getFormattedContent()) {
            this._updateContentTextArea();
        }
        return;
    }

    this.noteStore.getNoteContent(onSuccess, onFailure, this.user, this.note, this.noteFilter);
};

NoteViewAssistant.prototype.activate = function(event) {
    if (!EvernoteContext.getUpdateQueue().hasTicketForNote(this.note) && EvernoteContext.isConnected()) {
        Mojo.Log.info("[NoteView] Connected, refreshing");
        this._handleRefreshNoteCommand();
        //this.noteStore.listNotebooks(this.onNotebooksLoadSuccess.bind(this), this.onNotebooksLoadFailure.bind(this));
    } else {
        Mojo.Log.info("[NoteView] Not connected, using note itself");
        this.onNoteMetadataLoadSuccess(this.note);
        this._updateContentTextArea();
    }
};

NoteViewAssistant.prototype.deactivate = function(event) {
};

NoteViewAssistant.prototype.toggleDetailsWidget = function() {
	if(this.filesWidget.mojo.getOpenState() == true){
		this.filesWidget.mojo.toggleState();
	}
	this.detailsWidget.mojo.toggleState();
};

NoteViewAssistant.prototype.toggleFilesWidget = function() {
	if(this.detailsWidget.mojo.getOpenState() == true){
		this.detailsWidget.mojo.toggleState();
	}
	this.filesWidget.mojo.toggleState();
};

NoteViewAssistant.prototype.onNoteMetadataLoadSuccess = function(note) {
    var viewId = this.note.viewId;
	this._resetDetailContainerHeight();

    this.note.updatePropertiesBut(note, new Array("content","formattedContent","highlightedFormattedContent"));
    var title = note.title || $L("Untitled note");
    title = title.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
    this.titleValueLabel.update(title);

    this.createdValueLabel.update(note.getFormattedCreateDate());
    this.updatedValueLabel.update(note.getFormattedUpdateDate());
    this.tagListWidget.update(note.getTagNamesAsString());
	
	if(this.notebookContainer.offsetHeight > 26){
		this._updateNoteDetailContainerHeight(this.notebookValueLabel.offsetHeight);
	}
	if(this.tagsContainer.offsetHeight > 26){
		this._updateNoteDetailContainerHeight(this.tagListWidget.offsetHeight); 
	}
	
    var locationLink = "maploc:" + note.attributes.latitude + "," + note.attributes.longitude;
    this.locationLink.href = locationLink;
    if (note.attributes.longitude == 0 && note.attributes.latitude == 0) {
        this.gpsLocation.hide();
    }
    else {
        if (! this.geoNameRetrieved){
			this._retrieveGeoName(note);
		}
		this._updateNoteDetailContainerHeight(26);
    }

    this.note.viewId = viewId;

	var sourceLink = note.attributes.sourceURL;
	var urlTest=new RegExp("^(http|https)://.+");
	if(!urlTest.test(sourceLink)){
		sourceLink = "http://"+sourceLink;
	}
	this.sourceLink.href = sourceLink;
	
	if (note.attributes.sourceURL == null) {
        this.sourceContainer.hide(); 
    }
    else {
		this._updateNoteDetailContainerHeight(26);
    }
};

NoteViewAssistant.prototype._resetDetailContainerHeight = function() {
	this.noteDetailsContainer.style.height = '116px';
};

NoteViewAssistant.prototype._updateNoteDetailContainerHeight = function(height) {
	var newHeight = this.noteDetailsContainer.offsetHeight + height;
    this.noteDetailsContainer.style.height = newHeight + 'px';
};

NoteViewAssistant.prototype._retrieveGeoName = function(note) {
    if (! EvernoteContext.isConnected()) {
        return;
    }

    this.geoNameRetrieved = true;

    var url = "http://ws.geonames.org/citiesJSON";
    url += "?lang=en&username=evernote&maxRows=10";
    url += "&north=" + ( note.attributes.latitude + 0.1 );
    url += "&south=" + ( note.attributes.latitude - 0.1 );
    url += "&east=" + ( note.attributes.longitude + 0.1 );
    url += "&west=" + ( note.attributes.longitude - 0.1 );

    var self = this;
    new Ajax.Request(url, {
        method: 'get',
        onSuccess: function(transport) {
            var response = transport.responseJSON;
            if (response.geonames) {
                var geoname = response.geonames[0];
                var locationName = geoname.name + ", " + geoname.countrycode;
                self.locationLinkText.update(locationName);
            }
        },
        onFailure: function(transport) {
            Mojo.Log.warn("[Note View] Could not retrieve geoname");
            self.geoNameRetrieved = false;
        }
    });
};

NoteViewAssistant.prototype.onNoteContentLoadSuccess = function(transport) {
    this.note.highlightedFormattedContent = transport.responseText;
    this.note.todos = new Hash();
    this.note.unsyncedTodos = false;

    var f = (function(transport) {
        this.note.formattedContent = transport.responseText;
        this._updateContentTextArea();
    }).bind(this);

    if (this.noteFilter && this.noteFilter.hasWords()) {
        this.noteStore.getNoteContent(f, f.curry(transport), this.user, this.note, null);
    }
    else {
        f(transport);
    }
};

NoteViewAssistant.prototype.onNoteContentLoadFailure = function(response) {
    Mojo.Log.error("[Note View] Failed to load note content: " + this.note.title + "\n" + response);
    this._stopSpinner();
    EvernoteContext.getNotifier().alertDialog($L("Error"), $L("An error occurred while loading note"), $L("OK"), this.controller);
};

// NoteViewAssistant.prototype.onNotebooksLoadSuccess = function(notebooksJSON) {
//     var notebooks = notebooksJSON.list;
//     for (var i = 0; i < notebooks.length; i++) {
//         if (notebooks[i].guid == this.note.notebookGuid) {
//             this._stopSpinner();
//             this.notebookValueLabel.update(notebooks[i].name);
//             break;
//         }
//     }
// };
// 
// NoteViewAssistant.prototype.onNotebooksLoadFailure = function() {
//     EvernoteContext.getNotifier().alertDialog($L('Error'), $L('An error has occured while loading notebooks'), $L('OK'), this.controller);
// };

NoteViewAssistant.prototype.handleCommand = function(event) {
	//ADDED NEXT 4 LINES
	if (event.command == 'do-touchpad-back') {
		Mojo.Controller.getAppController().assistant.closeNoteStage(this.note);
		return;
	}
	
    if (event.type == Mojo.Event.command) {
        switch (event.command)
                {
            case 'do-note-refresh':
                this._handleRefreshNoteCommand();
                break;
        }
    }
};

NoteViewAssistant.prototype._handleRefreshNoteCommand = function() {
    this._startSpinner();

    var globalOnSuccess = this._stopSpinner.bind(this);
    
    var mc = new MultiCommand(globalOnSuccess, this.onNoteContentLoadFailure.bind(this));

    // note content
    var onNoteContentSuccess = (function(content) {
        this.onNoteContentLoadSuccess(content);
        mc.succeed();
    }).bind(this);
    var f1 = this._reloadNoteContent.bind(this,onNoteContentSuccess, mc.fail.bind(mc));
    
    // note metadata
    var onNoteMetadataSuccess = (function(note) {
        this.onNoteMetadataLoadSuccess(note);
        mc.succeed();
    }).bind(this);
    var f2 = this._reloadNoteMetadata.bind(this, onNoteMetadataSuccess, mc.fail.bind(mc));

    mc.async(f2, f1);
};

NoteViewAssistant.prototype._startSpinner = function() {
    try {
        this.spinner.mojo.start();
    }
    catch (e) {
        Mojo.Log.warn("[Note View] Error starting spinner: " + e);
    }
};

NoteViewAssistant.prototype._stopSpinner = function() {
    try {
        this.spinner.mojo.stop();
    }
    catch (e) {
        Mojo.Log.warn("[Note View] Error stopping spinner: " + e);
    }
};

NoteViewAssistant.prototype._updateContentTextArea = function() {
    this.deinitialToDoCheckboxes();
    this.deinitializeAttachments();
	this.deinitializeImages();
	this.deinitializeDecryption();
    this.contentTextArea.update(this.note.getHighlightedFormattedContent(true));
    if (this.note.hasFile()) {
        this.contentTextArea.insert("<img src=\"" + this.note.getFileFullPath() + "\" style=\"width:100%;\" />");
    }
    this.initialToDoCheckboxes.defer();
   this.initializeAttachments.defer();
	this.initializeImages.defer();
	this.initializeDecryption.defer();

	this.cmdMenuModel.items[0].disabled = false;
	this.cmdMenuModel.items[1].disabled = false;
	this.cmdMenuModel.items[2].disabled = false;
	
	this.controller.modelChanged(this.cmdMenuModel);
};

NoteViewAssistant.prototype._initializeToDoCheckboxes = function() {

    var unsyncedTodos = this.note.unsyncedTodos;
    var todoCheckboxes = this.contentTextArea.select("input.en-todo");
    var todoIndex = 0;
    var self = this;
    todoCheckboxes.each(function(element) {
        element.enable();
        Event.observe(element, "click", self.handleToDoCheckboxChange);
        var todo = "todo-" + todoIndex;
        element.writeAttribute("id", todo);
        if (unsyncedTodos) {
            element.checked = self.note.todos.get(todo);
        } else {
            self.note.todos.set(todo, element.checked);
        }

        todoIndex = todoIndex + 1;
    });

};

NoteViewAssistant.prototype._handleToDoCheckboxChange = function(event) {
    var element = event.findElement();
    var todo = element.readAttribute("id");
    this.note.todos.set(todo, element.checked);
    this.note.unsyncedTodos = true;
    EvernoteContext.getSession().NoteStore.updateTodos(function() {}, function() {},
            EvernoteContext.getSession().currentUser,
            this.note
            );
};

NoteViewAssistant.prototype._deinitializeToDoCheckboxes = function() {

    var todoCheckboxes = this.contentTextArea.select("input.en-todo");
    todoCheckboxes.each(function(element) {
        element.stopObserving();
    });

};

NoteViewAssistant.prototype._initializeAttachments = function() {
    Mojo.Log.info("[NoteView] Finding attachments in note content: %s", this.contentTextArea.innerHTML);
	this.attachmentsListModel.items = new Array();
    var attachments = this.contentTextArea.select("a[href^=\"" + EvernoteContext.getBaseURL() + "\"]");
    Mojo.Log.info("[NoteView] Found %i attachments.", attachments.length);
    var self = this;
    attachments.each(function(element) {
		Mojo.Log.info("[Noteview] attachment:", element);
        Event.observe(element, "click", self.handleAttachment);
		var text = element.innerText;
		var nameSizeSeparatorPattern = /(\r|\n)+/;
		var nameSizeSeparator = nameSizeSeparatorPattern.exec(text);
		var separator = nameSizeSeparator.index;
		var size = text.substring(separator+1);
		var name = text.substring(0, separator);
		self.attachmentsListModel.items.push({'title':name+" - "+size,
												'fileName':name,
												'href':element.href,
												'progress': 0,
											    'iconPath': "../images/list_button_add.png",
											    'status': Downloader.STATUS_NONE,
											    'buttonLabel': "Cancel",
											    'buttonClass': "negative"});
    });
	var media = this.contentTextArea.select("embed[src^=\"" + EvernoteContext.getBaseURL() + "\"]");
	Mojo.Log.info("[NoteView] Found %i media files.", media.length);
    media.each(function(element) {
		Mojo.Log.info("[Noteview] media file:", element.readAttribute("fileName"));
        self.attachmentsListModel.items.push({'title':element.readAttribute("fileName")+' - '+element.readAttribute("filesize"), 
												'fileName':element.readAttribute("fileName"),
												'href':element.readAttribute("src"),
												'progress': 0,
											    'iconPath': "../images/list_button_add.png",
											    'status': Downloader.STATUS_NONE,
											    'buttonLabel': "Cancel",
											    'buttonClass': "negative"});
    });
	
	if((media.length + attachments.length) > 0){
		var newHeight = (media.length + attachments.length) * 50;
		this.controller.get('filesContainer').style.height = newHeight + 'px';
	}
	else{
		this.controller.get('filesButton').hide();
	}
	this.controller.modelChanged(this.attachmentsListModel);
};

NoteViewAssistant.prototype._initializeImages = function() {
    Mojo.Log.info("[NoteView] Finding images in note content: %s", this.contentTextArea.innerHTML);
    this.images = this.contentTextArea.select("img[src^=\"" + EvernoteContext.getResourceBaseURL() + "\"]");
    Mojo.Log.info("[NoteView] Found %i image(s).", this.images.length);
    var self = this;
    this.images.each(function(element) {
        Event.observe(element, "click", self.handleImage);
    });
};

NoteViewAssistant.prototype._handleImage = function(event) {
    Mojo.Log.info("[NoteView] Image for %s clicked!", event.target.src);
    event.stop();
	var src = event.target.src;
	var queryMarker = src.indexOf("?");
	var isInk = src.indexOf("\.ink?");
	Mojo.Log.info("--------------> this is a ink note? :" + isInk);
	var lowRes = '';
	if (queryMarker != -1 && isInk == -1){
		lowRes = src;
		src = src.substring(0, queryMarker);
	}
    Mojo.Log.info("lowRes picture = " + lowRes);
	this.controller.stageController.pushScene('imageview', {'heighRes':src, 'lowRes':lowRes});
};

NoteViewAssistant.prototype._initializeDecryption = function() {
    Mojo.Log.info("[NoteView] Finding encryptedParagraphs in note content: %s", this.contentTextArea.innerHTML);
    this.encryptedParagraphs = this.contentTextArea.select("img[name^=\"en-crypt\"]");
    Mojo.Log.info("[NoteView] Found %i encryptedParagraph(s).", this.encryptedParagraphs.length);
    var self = this;
    this.encryptedParagraphs.each(function(element, index) {
        Event.observe(element, "click", self._handleDecryption.bind(self, index));
    });
};

NoteViewAssistant.prototype._deinitializeDecryption = function() {
    Mojo.Log.info("[NoteView] Stop observing encryptedParagraphs.");
    if(this.encryptedParagraphs){
	    Mojo.Log.info("[NoteView] Found %i encryptedParagraph(s).", this.encryptedParagraphs.length);
	    this.encryptedParagraphs.each(function(element) {
	        element.stopObserving();
	    });
	}
};

NoteViewAssistant.prototype._handleDecryption = function(index, event) {
    Mojo.Log.info("[NoteView] Encrypted section %s clicked!", event.item);
    event.stop();
	var hint = event.currentTarget.title;
	this.enc = event.currentTarget.alt;
	this.index = index;
   	this.controller.showDialog({
			template: 'note-view/passwordDialog-scene',
			assistant: new PasswordDialogAssistant(this, this.decryption.bind(this), hint),
			preventCancel: true
		});
    
};

NoteViewAssistant.prototype.decryption = function(value) {
	if(!this.noteDecryptor){
		this.noteDecryptor = new NoteDecryptor();
	}
	var dec = this.noteDecryptor.decrypt(this.enc, value);
	Mojo.Log.info("[NoteView] Decripted text: ", dec);
	if(dec){
		dec = "<span style='background-color:#B8E281;'>" + dec + "</span>";
		var adjacentNode = this.controller.window.document.getElementsByName("en-crypt")[this.index];
	    var range = this.controller.window.document.createRange();
	    range.selectNode(adjacentNode);
	    var frag = range.createContextualFragment(dec);
	    adjacentNode.parentNode.insertBefore(frag, adjacentNode);
	    adjacentNode.hide();
	}
	//else{
	// 		Mojo.Controller.errorDialog($L("The note could not be decrypted. The password was probably wrong."));
	// 	}
};

NoteViewAssistant.prototype._handleAttachment = function(event) {
    Mojo.Log.info("[NoteView] Attachment for %s clicked!", event.item.title);
    
    event.stop();

	var href = event.item.href;
    var fileName = event.item.fileName;
	if(this.download.filenName == event.item.fileName){
		return;
	}
	this.download = event.item;
	if(this.download.status == Downloader.STATUS_COMPLETE){
		this._openFile();
		return;
	}
    var onChoose = (function(value) {
        if (value != "yes") {
            return;
        }

        if(this.download.status == Downloader.STATUS_DOWNLOADING) {
            var startDownload = this._startDownload.bind(this,href,fileName);
            EvernoteContext.getDownloader().cancelDownload(startDownload,startDownload, this.download.ticket);    
        } else {
            this._startDownload(href, fileName);
        }
    }).bind(this);


    var choices = [ { label: $L("Download"), value: "yes", type: "affirmative" },
                    { label: $L("Cancel"), value: "no", type: "dismiss" } ];
    var msg = $L("Do you want to download '#{fileName}'?").interpolate({fileName: fileName});

    if (this.download.status == Downloader.STATUS_DOWNLOADING) {
          var fileNames = {downloadFileName: this.download.fileName, fileName: fileName};
          choices = [ { label: $L("Download  #{fileName}").interpolate(fileNames), value: "yes", type: "negative" },
                          { label: $L("Keep downloading #{downloadFileName}").interpolate(fileNames), value: "no", type: "dismiss" } ];
          msg = $L("You are already downloading '#{downloadFileName}. Do you wish to cancel and download #{fileName} instead'?").interpolate(fileNames);
    }
  
    EvernoteContext.getNotifier().alertForm($L("Alert"), msg, onChoose, choices);
    
};

NoteViewAssistant.prototype._startDownload = function(href,fileName) {
    EvernoteContext.getDownloader().startDownload(href, fileName, this.note, this.updateDownload.bind(this), this.downloadFailed.bind(this));
    //this.downloadPanelModel.open = true;
    this.controller.modelChanged(this.downloadPanelModel);
};

NoteViewAssistant.prototype._deinitializeImages = function() {
    Mojo.Log.info("[NoteView] Stop observing Image links.");
    if(this.images){
	    Mojo.Log.info("[NoteView] Found %i image(s).", this.images.length);
	    this.images.each(function(element) {
	        element.stopObserving();
	    });
	}
};

NoteViewAssistant.prototype._deinitializeAttachments = function() {
    Mojo.Log.info("[NoteView] Stop observing attachment links.");
    var attachments = this.contentTextArea.select("a[href^=\"" + EvernoteContext.getBaseURL() + "\"]");
    Mojo.Log.info("[NoteView] Found %i attachments.", attachments.length);
    attachments.each(function(element) {
        element.stopObserving();
    });
};

NoteViewAssistant.prototype._openFile = function() {
    if(this.download.status == Downloader.STATUS_COMPLETE) {
        this.controller.serviceRequest('palm://com.palm.applicationManager', {
            method: 'open',
            parameters: {
                target: this.download.file
            }
        });
    } 
};

NoteViewAssistant.prototype.updateDownload = function(download) {
    this.download.ticket = download.ticket;
    this.download.status = download.status;

    if(download.status == Downloader.STATUS_COMPLETE) {
        this.download.progress = 1;
        this.download.title = $L("Tap to open '#{fileName}'").interpolate(download);
        this.download.file = download.file;
    } else {
        this.download.progress = download.progress;
        this.download.title = $L("Downloading '#{fileName}").interpolate(download);
    }

    this.controller.modelChanged(this.download);
};

NoteViewAssistant.prototype.downloadFailed = function(download) {
    this.download.ticket = download.ticket;
    this.download.status = download.status;
    this.download.title = $L("Failed to download #{fileName}").interpolate(download);

    this.controller.modelChanged(this.download);
};