function PreferencesAssistant () {
	this.isDirty = false;
	this.prefs = null;
}

PreferencesAssistant.prototype.setup = function() {

  var sortOrderListAttributes = {
	label: $L("Order"),
	labelPlacement:Mojo.Widget.labelPlacementLeft,
    choices: [
      { label: $L("Created"), value: Evernote.sortorder.created },
      { label: $L("Updated"), value: Evernote.sortorder.updated },
 	  { label: $L("Title"), value: Evernote.sortorder.title }]
  };
  this.sortOrderListModel = {
    value: null,
    disabled: false
  };
  this.sortOrderList = this.controller.setupWidget('sort-order-list', sortOrderListAttributes, this.sortOrderListModel);
  this.controller.listen('sort-order-list', Mojo.Event.propertyChange, this._setDirty.bind(this));

  var ascendingListAttributes = {
	label: $L("Direction"),
	labelPlacement:Mojo.Widget.labelPlacementLeft,
    choices: [
      { label: $L("Ascending"), value: 1 },
      { label: $L("Descending"), value: 0 }]
  };
  this.ascendingListModel = {
    value: null,
    disabled: false
  };
  this.ascendingList = this.controller.setupWidget('ascending-list', ascendingListAttributes, this.ascendingListModel);
  this.controller.listen('ascending-list', Mojo.Event.propertyChange, this._setDirty.bind(this));
	
  this.thumbnailsToggleModel = { value: true };
  this.thumbnailsToggle = this.controller.setupWidget('thumbnails-toggle', null, this.thumbnailsToggleModel );
  this.controller.listen('thumbnails-toggle', Mojo.Event.propertyChange, this._setDirty.bind(this));

  this.requireWifiToggleModel = { value: null };
  this.requireWifiToggle = this.controller.setupWidget('require-wifi-toggle', null, this.requireWifiToggleModel );
  this.controller.listen('require-wifi-toggle', Mojo.Event.propertyChange, this._saveChanges.bind(this));

  EvernoteContext.getPrefs( this._setupScene.bind(this), this._loadFailure.bind(this));

  var menuManager = EvernoteContext.getMenuManager();
  var appMenuModel = menuManager.getBaseAppMenuModel();
  menuManager.removePreferencesCommand(appMenuModel);
  menuManager.setupAppMenu(this.controller, appMenuModel);

    this.controller.topContainer().select(".localizable").each(function(elem) {
        elem.innerHTML = $L(elem.innerHTML);
    });
    
};

PreferencesAssistant.prototype._setupScene = function(prefs) {

  	if(!prefs){
		Mojo.Log.warn("[PreferencesAssistant] _setupScene: Preferences not found, using default.");
		prefs = new Preferences();
	}
	this.prefs = prefs;
  this.sortOrderListModel.value = prefs.sortOrder;
  this.controller.modelChanged(this.sortOrderListModel);
	if(typeof prefs.ascending != "undefined" && prefs.ascending){
		this.ascendingListModel.value = 1;
	}else {
		this.ascendingListModel.value = 0;
	}
  this.controller.modelChanged(this.ascendingListModel);

  this.thumbnailsToggleModel.value = prefs.showThumbnails;
  this.controller.modelChanged(this.thumbnailsToggleModel);

  // this.notebookListModel.value = prefs.filterNotebook;
  //   this.controller.modelChanged(this.notebookListModel);

  this.requireWifiToggleModel.value = prefs.requireWiFi;
  this.controller.modelChanged(this.requireWifiToggleModel);

};

PreferencesAssistant.prototype.handleCommand = function(event) {
	//ADDED NEXT 4
	if (event.command == 'do-touchpad-back') {
		this.controller.stageController.popScene({'isDirty':this.isDirty, 'prefsReturn':true});
		return;
	}
	if (event.type === Mojo.Event.back) {
		this.controller.stageController.popScene({'isDirty':this.isDirty, 'prefsReturn':true});
		return;
	}	
};

PreferencesAssistant.prototype._setDirty = function() {
	if(this.prefs){
		if(this.prefs.sortOrder === this.sortOrderListModel.value &&
			this.prefs.showThumbnails === this.thumbnailsToggleModel.value &&
			((this.prefs.ascending == true && this.ascendingListModel.value == 1) ||
			(this.prefs.ascending == false && this.ascendingListModel.value == 0))){
				this.isDirty = false;
			}
			else{
				this.isDirty = true;
			}
	}
	this._saveChanges();
};

PreferencesAssistant.prototype._saveChanges = function() {
  var prefs = new Preferences();
	if(this.ascendingListModel.value == 1){
		prefs.ascending = true;
	}else {
		prefs.ascending = false;
	}
  prefs.sortOrder = this.sortOrderListModel.value;
  prefs.requireWiFi = this.requireWifiToggleModel.value;
  prefs.showThumbnails = this.thumbnailsToggleModel.value;

  var success = function() {
    EvernoteContext.getNotifier().banner($L("Saved preferences."), "Preferences");
  };

  var failure = function() {
    EvernoteContext.getNotifier().banner($L("Error saving preferences."), "Preferences");
  };

  EvernoteContext.setPrefs(prefs, success, failure);
};

PreferencesAssistant.prototype._loadFailure = function(error) {
  EvernoteContext.getNotifier().alertDialog($L("Error"), $L("An error was encountered while retrieving your preferences: ") + error, $L("OK"), this.controller);
};

PreferencesAssistant.prototype.activate = function() {
};

PreferencesAssistant.prototype.deactivate = function() {
};

PreferencesAssistant.prototype.cleanup = function(){
    this.controller.stopListening('sort-order-list', Mojo.Event.propertyChange, this._saveChanges.bind(this));
    this.controller.stopListening('require-wifi-toggle', Mojo.Event.propertyChange, this._saveChanges.bind(this));
	this.controller.stopListening('ascending-list', Mojo.Event.propertyChange, this._saveChanges.bind(this));
	this.controller.stopListening('thumbnails-toggle', Mojo.Event.propertyChange, this._saveChanges.bind(this));
};